#include "stdafx.h"
#include "LeoHelpers_String.h"
#include "LeoHelpers_Scope.h"
#include "LeoHelpers_File.h"
#include "LeoHelpers_Debug.h"

#ifdef LEOHELPERS_ENABLE_DEBUG_TO_FILE

static LeoHelpers::MutexWrapper S_mxOutputFile(L"LeoHelpers_File_Debug_1DD424C6-7999-449D-BBCB-98D235CF189E");
static std::wstring S_strOutputPath;

void LeoHelpers::SendDebugOutputToFile(const wchar_t *szFilepath)
{
	LeoHelpers::MutexScoper mxs(S_mxOutputFile);

	if (szFilepath)
	{
		S_strOutputPath = szFilepath;
	}
	else
	{
		S_strOutputPath.clear();
	}
}

#endif // LEOHELPERS_ENABLE_DEBUG_TO_FILE


void LeoHelpers::OutputDebug(const wchar_t *szPrefix, const wchar_t *szClass, const wchar_t *szMethod, const wchar_t *szMessage)
{
	DWORD dwErr = ::GetLastError();

	wchar_t szThreadId[32];

	DWORD dwThreadId = GetCurrentThreadId();
	_snwprintf_s(szThreadId, _countof(szThreadId), _TRUNCATE, L"[Thread %ld] ", dwThreadId);

	std::wstring strMsg = szThreadId;

	if (szPrefix != NULL)
	{
		strMsg += szPrefix;
	}

	if (szClass != NULL)
	{
		strMsg += szClass;

		if (szMethod != NULL)
		{
			strMsg += L"::";
		}
		else
		{
			strMsg += L": ";
		}
	}

	if (szMethod != NULL)
	{
		strMsg += szMethod;
		strMsg += L": ";
	}

	if (szMessage != NULL)
	{
		strMsg += szMessage;
	}

	while(!strMsg.empty() && iswspace(strMsg[strMsg.length() - 1]))
	{
		strMsg.resize(strMsg.length() - 1);
	}

	strMsg += L"\r\n";

#ifdef LEOHELPERS_ENABLE_DEBUG_TO_FILE
	// If sending output to a file, we re-open it for every line. This isn't efficient. It's for debug so it doesn't need to be.

	bool bDoneOutput = false;

	// scope
	{
		LeoHelpers::MutexScoper mxs(S_mxOutputFile);

		if (!S_strOutputPath.empty())
		{
			HANDLE hFile = ::CreateFile(S_strOutputPath.c_str(), GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
			if (hFile != INVALID_HANDLE_VALUE)
			{
				if (::SetFilePointerEx(hFile, LARGE_INTEGER{0,0}, NULL, FILE_END))
				{
					wchar_t szDate[128] = {0};
					wchar_t szTime[128] = {0};
					wchar_t szModulePath[MAX_PATH+1] = {0};
					
					SYSTEMTIME st;
					GetSystemTime(&st);
					if (::GetModuleFileName(NULL, szModulePath, _countof(szModulePath)-1)
					&&	::GetDateFormat(LOCALE_INVARIANT, DATE_SHORTDATE, &st, NULL, szDate, _countof(szDate)-1)
					&&	::GetTimeFormat(LOCALE_INVARIANT, TIME_FORCE24HOURFORMAT, &st, NULL, szTime, _countof(szTime)-1))
					{
						std::wstring strFileLine = L"[";
						strFileLine += szDate;
						strFileLine += L" ";
						strFileLine += szTime;
						if (szModulePath)
						{
							strFileLine += L" | ";
							strFileLine += LeoHelpers::GetLastPathPart(szModulePath);
						}
						strFileLine += L"] -- ";
						strFileLine += strMsg;

						DWORD dwDontCare = 0;
						if (::WriteFile(hFile, strFileLine.c_str(), static_cast< DWORD >( strFileLine.length() * sizeof(wchar_t) ), &dwDontCare, NULL))
						{
							bDoneOutput = true;
						}
					}
				}
				::CloseHandle(hFile);
			}
		}
	}

	if (!bDoneOutput)
#endif
	{
		::OutputDebugString(strMsg.c_str());
	}

	::SetLastError(dwErr);
}


#ifdef _DEBUG

// static
unsigned __stdcall LeoHelpers::LeoAssert::staticThread(void *pVoidThis)
{
	LeoAssert &la = *reinterpret_cast< LeoAssert * >(pVoidThis);

	if (S_OK != ::CoInitialize(NULL))
	{
		return 0;
	}

	std::wstring strMsg;
	LeoHelpers::StringFormatToString(&strMsg, L"DEBUG ASSERTION FAILED\n\nCondition: %s\n\nFile: %s\nLine: %d\n\nBreak?", la.m_szExpression, la.m_szFile, la.m_iLine);

	wchar_t szModulePath[MAX_PATH+1] = {0};
	GetModuleFileName(NULL, szModulePath, _countof(szModulePath)-1);
	szModulePath[_countof(szModulePath)-1] = L'\0';

	if (IDYES == ::MessageBox(NULL, strMsg.c_str(), LeoHelpers::GetLastPathPart(szModulePath), MB_YESNO|MB_ICONEXCLAMATION|MB_TOPMOST))
	{
		la.m_bBreak = true;
	}

	::CoUninitialize();

	return 0;
}

//static
void LeoHelpers::LeoAssert::Show(const wchar_t *szExpression, const wchar_t *szFile, int iLine)
{
	DWORD dwErr = ::GetLastError();

	LeoAssert la(szExpression, szFile, iLine);

	unsigned int uiIgnored = 0;

	HANDLE hThread = reinterpret_cast<HANDLE>( ::_beginthreadex(NULL, 0, staticThread, &la, 0, &uiIgnored) );

	if (hThread != NULL)
	{
		::WaitForSingleObject(hThread, INFINITE);
		::CloseHandle(hThread);
	}

	if (la.m_bBreak)
	{
		::_CrtDbgBreak();
	}

	::SetLastError(dwErr);
}

#endif // _DEBUG

const wchar_t *LeoHelpers::DebugHexDumper::GetNextLine()
{
	if (m_dataSize == 0)
	{
		return NULL;
	}

	//  00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001111111111111
	//  00000000001111111111222222222233333333334444444444555555555566666666667777777777888888888899999999990000000000111
	//  01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012
	// "Data: xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx  aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaN"

	leoAssert(_countof(m_szLineBuffer) == 112);

	m_szLineBuffer[0] = L'D';
	m_szLineBuffer[1] = L'a';
	m_szLineBuffer[2] = L't';
	m_szLineBuffer[3] = L'a';
	m_szLineBuffer[4] = L':';
	for (size_t i = 5; i < 111; ++i)
		m_szLineBuffer[i] = L' ';
	m_szLineBuffer[111] = L'\0';

	// This is a fairly inefficient way to generate the hex dump but it's only for debugging so that's fine.
	for (size_t x = 0; x < 32 && m_dataSize > 0; ++x, --m_dataSize, ++m_pByteData)
	{
		wchar_t *szBufX = m_szLineBuffer +  6 + x*2 + x/4;
		wchar_t *szBufA = m_szLineBuffer + 79 + x;
		wchar_t cX = szBufX[2];
		wchar_t cA = szBufA[1];
		swprintf_s(szBufX, 3, L"%02x", static_cast<unsigned int>( *m_pByteData ));
		swprintf_s(szBufA, 2, L"%hc",  isprint(*m_pByteData) ? *m_pByteData : '.');
		szBufX[2] = cX;
		szBufA[1] = cA;
	}

	return m_szLineBuffer;
}
